import type {paths, components} from './schema';
import createClient from 'openapi-fetch';
import {fetchEventSource} from '@microsoft/fetch-event-source';
import {token} from '../store';
import { replace } from 'svelte-spa-router';
import { writable } from 'svelte/store';

export type Session = components['schemas']['de.mattv.fileserver.Response$Session'];
export type Node = components['schemas']['de.mattv.fileserver.Response$Node'];
export type PathSegment = components['schemas']['de.mattv.fileserver.Response$PathSegment'];
export type CreateNodeInfo = components['schemas']['de.mattv.fileserver.Response$CreateNodeInfo'];
export type UserInfo = components['schemas']['de.mattv.fileserver.Response$UserInfo'];
export type AllPaths = keyof paths

export enum RpcClientType { AUTH, SHARE_RO, SHARE_RW }

const _getRpcClient = () => {
    let obj = {
        token: '',
        client: createClient<paths>(),
        ty: writable(RpcClientType.AUTH),

        signup: (username: string, password: string) =>
            obj.client.POST('/api/public/auth/signup', { body: { username, password } }).then(v => v.data),
        login: (username: string, password: string, otp?: string) =>
            obj.client.POST('/api/public/auth/login', { body: { username, password, otp } }).then(v => v.data),

        send_recovery_key: (username: string) =>
            obj.client.POST('/api/public/auth/send_recovery_key', { body: username }).then(v => v.data),
        reset_password: (key: string, password: string) =>
            obj.client.POST('/api/public/auth/reset_password', { body: { key, password } }).then(v => v.data),


        change_password: (oldPassword: string, newPassword: string) =>
            obj.client.POST('/api/user/auth/change_password', { body: { oldPassword, newPassword } }).then(v => v.data),

        logout: () => obj.client.POST('/api/user/auth/logout').then(v => v.data),
        logoutAll: () => obj.client.POST('/api/user/auth/logout_all').then(v => v.data),
        deleteAccount: () => obj.client.POST('/api/user/auth/delete').then(v => v.data),
        sessionInfo: () => obj.client.POST('/api/user_share/session').then(v => v.data),

        tfaSetupMail: () => obj.client.POST('/api/user/tfa/setup_mail').then(v => v.data),
        tfaSetupTotp: () => obj.client.POST('/api/user/tfa/setup_totp').then(v => v.data),
        tfaComplete: (code: string) => obj.client.POST('/api/user/tfa/complete', { body: code }).then(v => v.data),
        tfaDisable: () => obj.client.POST('/api/user/tfa/disable').then(v => v.data),


        getNode: (node: number) => obj.client.POST('/api/user_share/fs/node', { body: node }).then(v => v.data),
        getPath: (node: number) => obj.client.POST('/api/user_share/fs/path', { body: node }).then(v => v.data),
        getNodesSize: (nodes: number[]) => obj.client.POST('/api/user_share/fs/size', { body: nodes }).then(v => v.data),
        getMime: (node: number) => obj.client.POST('/api/user_share/fs/mime', { body: node }).then(v => v.data),
        downloadPreview: (node: number) => obj.client.POST('/api/user_share/fs/preview', { body: node }).then(v => v.data),

        createNode: (name: string, parent: number, file: boolean) =>
            obj.client.POST('/api/user_share/fs/create', { body: { name, parent, file } }).then(v => v.data),

        deleteNodes: (nodes: number[], cbk: (v: string|null) => void) => fetchEventSource('/api/user/fs/delete', {
            method: 'POST',
            body: JSON.stringify(nodes),
            headers: {
                'Authorization': 'Bearer ' + obj.token,
                'Content-Type': 'application/json'
            },
            onmessage: v => cbk(v.data),
            onerror: e => { throw e; },
            onclose: () => cbk(null)
        }),

        shareNode: (node: number) => obj.client.POST('/api/user/fs/share', { body: node }).then(v => v.data),
        unshareNode: (node: number) => obj.client.POST('/api/user/fs/unshare', { body: node }).then(v => v.data),
        shareSetPw: (node: number, pw?: string) => obj.client.POST('/api/user/fs/share_pw', { body: { id: node, password: pw } }).then(v => v.data),
        shareUnlock: (pw: string) => obj.client.POST('/api/user_share/share_unlock', { body: pw }).then(v => v.data),

        admin: {
            listUsers: () => obj.client.POST('/api/admin/users').then(v => v.data),
            setEnabled: (id: number, state: boolean) => obj.client.POST('/api/admin/user/set_enabled', { body: { id, state } }).then(v => v.data),
            setAdmin: (id: number, state: boolean) => obj.client.POST('/api/admin/user/set_admin', { body: { id, state } }).then(v => v.data),
            sudo: (id: number) => obj.client.POST('/api/admin/user/sudo', { body: id }).then(v => v.data),
            logout: (id: number) => obj.client.POST('/api/admin/user/logout', { body: id }).then(v => v.data),
            disableTfa: (id: number) => obj.client.POST('/api/admin/user/disable_tfa', { body: id }).then(v => v.data),
            deleteUser: (id: number) => obj.client.POST('/api/admin/user/delete', { body: id }).then(v => v.data),
            unSudo: () => obj.client.POST('/api/admin/un_sudo').then(v => v.data),
            shutdown: () => obj.client.POST('/api/admin/shutdown').then(v => v.data),
        }
    };

    obj.client.use({
        onRequest({ schemaPath, request }) {
            if (schemaPath.startsWith('/api/public') || obj.token == '')
                return;

            request.headers.set('Authorization', `Bearer ${obj.token}`);
            return request;
        }
    });

    return obj;
}

//export type RpcClient = Omit<ReturnType<typeof _getRpcClient>, 'share_root'> & { share_root: null | number };
export type RpcClient = ReturnType<typeof _getRpcClient>;
export const getRpcClient: () => RpcClient = () => _getRpcClient();
export const globalRpc = getRpcClient();

globalRpc.client.use({
    onResponse({ schemaPath, response }) {
        if (schemaPath.startsWith('/api/public') || schemaPath == '/api/user/session')
            return response;
        if (response.status >= 400 && response.status != 404) {
            token.set(null);
            replace('/login');
        }
        return response;
    }
})
