package de.mattv.fileserver.routes;

import de.mattv.fileserver.Utils;
import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.Token;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.routes.fs.FsUtils;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.UserOrShareRestController;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.FileCopyUtils;
import org.springframework.util.FileSystemUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.util.concurrent.atomic.AtomicLong;

@Slf4j
@UserOrShareRestController
public class Upload {
    private static final File TEMP_DIR = new File("temp");
    private static final AtomicLong NEXT_TEMP_ID = new AtomicLong(0);

    @PostConstruct
    private void init() {
        if (TEMP_DIR.exists()) FileSystemUtils.deleteRecursively(TEMP_DIR);
        if (!TEMP_DIR.mkdir()) Utils.crash(log, "Failed to create temp directory");
    }

    @PreDestroy
    private void destroy() {
        FileSystemUtils.deleteRecursively(TEMP_DIR);
    }

    private File getTempFile() {
        return new File(TEMP_DIR, String.valueOf(NEXT_TEMP_ID.getAndIncrement()));
    }

    @PostMapping("/upload/{id}")
    private void upload(@AuthUser Token token, @PathVariable long id, HttpServletRequest request, HttpServletResponse response) throws IOException {
        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(id);
            if (node == null || !node.isFile) {
                response.sendError(400, "Invalid node");
                return;
            }
            if (token.isShare()) {
                if (!token.isShareRw()) {
                    response.sendError(401, "Share is not unlocked");
                    return;
                }
                if (!FsUtils.shareHasAccess(token, node)) {
                    response.sendError(401, "No access to node");
                    return;
                }
            }

            File tempFile = getTempFile();
            FileCopyUtils.copy(request.getInputStream(), Files.newOutputStream(tempFile.toPath()));

            node.size = 0;
            node.removePreview();
            Files.move(tempFile.toPath(), node.file.toPath(), StandardCopyOption.REPLACE_EXISTING);
            node.size = node.file.length();

            node.createThumbnail();
        }
    }
}
