package de.mattv.fileserver.routes.fs;

import de.mattv.fileserver.Response;
import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.Token;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.UserOrShareRestController;
import de.mattv.fileserver.util.NonNull;
import org.springframework.http.MediaTypeFactory;
import org.springframework.util.MimeType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

@UserOrShareRestController
public class Info {
    @PostMapping("/fs/node")
    private @NonNull Optional<Response.Node> nodeInfo(@AuthUser Token token, @RequestBody @NonNull long id) {
        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(id);
            if (node == null || !FsUtils.shareHasAccess(token, node)) return Optional.empty();

            Long shareRoot = token.getShareRoot();

            Response.Node me = Response.Node.from(
                    node,
                    (shareRoot == null) || (shareRoot != node.id),
                    node.children.stream()
                            .map(n -> Response.Node.from(n, true, new Response.Node[0]))
                            .toArray(Response.Node[]::new)
            );
            return Optional.of(me);
        }
    }

    @PostMapping("/fs/path")
    private @NonNull Optional<List<Response.PathSegment>> path(@AuthUser Token token, @RequestBody @NonNull long id) {
        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(id);
            if (node == null || !FsUtils.shareHasAccess(token, node)) return Optional.empty();

            ArrayList<Node> segments = new ArrayList<>();
            while (node != null) {
                segments.add(node);
                node = node.parent;
            }
            Long shareRoot = token.getShareRoot();
            if (shareRoot != null) {
                while (!segments.isEmpty() && segments.getLast().id != shareRoot) {
                    segments.removeLast();
                }
            }
            return Optional.of(
                    segments.stream()
                            .map(n -> new Response.PathSegment(n.name, n.isFile ? null : n.id))
                            .toList()
                            .reversed()
            );
        }
    }

    @PostMapping("/fs/size")
    private long size(@AuthUser Token token, @RequestBody @NonNull long[] ids) {
        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            if (token.isShare()) {
                ids = Arrays.stream(ids)
                        .mapToObj(user.nodes::get)
                        .filter(n -> (n != null) && (FsUtils.shareHasAccess(token, n)))
                        .mapToLong(n -> n.id)
                        .toArray();
            }
            return FsUtils.nodesSize(user, ids);
        }
    }

    @PostMapping("/fs/mime")
    private @NonNull Optional<String> mime(@AuthUser Token token, @RequestBody @NonNull long id) {
        User user = token.getUser();
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node node = user.nodes.get(id);
            if (node == null || !node.isFile || !FsUtils.shareHasAccess(token, node)) return Optional.empty();
            return MediaTypeFactory.getMediaType(node.name).map(MimeType::toString);
        }
    }
}
